
function [nominal_perf,worstcase_perf,stability]=robustnessanalysis(typemodel,vector_order,Ts,theta_hat,K_hat,chi,Ptheta,vec_w)


controller_valid=K_hat;

if strcmp(typemodel,'arx')
    [Fu,Fe,Gi,Hi,gradientG,ZNtf,ZDtf]=determinationFuFe_arx(theta_hat,vector_order,Ts);
elseif strcmp(typemodel,'armax')
    [Fu,Fe,Gi,Hi,gradientG,ZNtf,ZDtf]=determinationFuFe_armax(theta_hat,vector_order,Ts);
elseif strcmp(typemodel,'oe')
    [Fu,Fe,Gi,Hi,gradientG,ZNtf,ZDtf]=determinationFuFe_oe(theta_hat,vector_order,Ts);
elseif strcmp(typemodel,'bj')
    [Fu,Fe,Gi,Hi,gradientG,ZNtf,ZDtf]=determinationFuFe_bj(theta_hat,vector_order,Ts);
elseif strcmp(typemodel,'fir')
    [Fu,Fe,Gi,Hi,gradientG,ZNtf,ZDtf]=determinationFuFe_fir(theta_hat,vector_order,Ts);
else
    error('Not a valid model structure');
end;


Cid=ssbal(ss(controller_valid));

R=inv(Ptheta)/chi;
[UU,SV,VV]=svd(R);
SVbis=sqrt(SV);
TT=SVbis*VV';

%%%%%% NOMINAL PERFORMANCE

Sid=inv(1+Cid*Gi);
Sid=ssbal(Sid);
CSid=Cid*Sid;
CSid=minreal(ssbal(CSid),1e-7);
Tid=Gi*CSid;
Tid=minreal(ssbal(Tid),1e-7);
GSid=Gi*Sid;
GSid=minreal(ssbal(GSid),1e-7);

nominal_perf=[vec_w, abs(frxav(Tid,vec_w)).' , abs(frxav(CSid,vec_w)).' , abs(frxav(GSid,vec_w)).' , abs(frxav(Sid,vec_w)).'];


%%%%%%%%%% STABILITY VERIFICATION

vec_muw=[];

for jjj=1:length(vec_w),
    
puls=vec_w(jjj);
zn=frxav(ZNtf,puls);
zd=frxav(ZDtf,puls);
X=frxav(controller_valid,puls); 



MM_D=(-(zd+(X*zn))*inv(TT))/(1+(zd+(X*zn))*theta_hat);

MMre=real(MM_D);
MMim=imag(MM_D);
if max(MMim)<1e-10
muw=sqrt(MMre*MMre');
else
muw=sqrt((MMre*MMre')-(((MMre*MMim')^2)/(MMim*MMim')));
end;

vec_muw=[vec_muw;muw];

end;

stability=[vec_w,vec_muw];

if max(vec_muw)>1,

error('The controller is not robustly stable'); 
end;

%%%%%%%%%% COMPUTATION WORST CASE GS 

Wl1=1; Wl2=0; Wr1=0; Wr2=1;

vecGSw=[]; vecTw=[];

for jjj=1:length(vec_w),
    
puls=vec_w(jjj);
zn=frxav(ZNtf,puls);
zd=frxav(ZDtf,puls);
X=frxav(controller_valid,puls); 


    
z1=X*zn+zd;
Q=inv(X'*X*Wr1'*Wr1+Wr2'*Wr2);

a11c=real(zd'*zd*Wl2'*Wl2+zn'*zn*Wl1'*Wl1);
a11g=real(-Q*z1'*z1);
a11t=-R;

a12c=real(zd'*Wl2'*Wl2);
a12g=real(-Q*z1');
a12t=R*theta_hat;

a22c=real(Wl2'*Wl2);
a22g=real(-Q);
a22t=1-theta_hat'*R*theta_hat;

setlmis([]);
[gamma,ngamma,sgamma]=lmivar(1,[1 0]);
[tau,ntau,stau]=lmivar(1,[1 0]);




lmiterm([-1 1 1 tau],1,1);

lmiterm([-2 1 1 gamma],1,1);

lmiterm([3 1 1 0],a11c);
lmiterm([3 1 1 gamma],a11g,1);
lmiterm([3 1 1 tau],a11t,1); 

lmiterm([3 1 2 0],a12c);
lmiterm([3 1 2 gamma],a12g,1);
lmiterm([3 1 2 tau],a12t,1);

lmiterm([3 2 2 0],a22c);
lmiterm([3 2 2 gamma],a22g,1);
lmiterm([3 2 2 tau],a22t,1);

kappaopt=getlmis;

c=[1;0];


flag=1;
options=[1e-12,1000,1e12,10,flag];
[coutopt,xopt]=mincx(kappaopt,c,options);

if isempty(xopt),
gammaopt=1e12;
else
gammaopt=dec2mat(kappaopt,xopt,1);
end;

vecGSw=[vecGSw;sqrt(gammaopt)];

vecTw=[vecTw;abs(X)*sqrt(gammaopt)];

end;    

%%%%%%%%%% COMPUTATION WORST CASE S 

Wl1=0; Wl2=1; Wr1=0; Wr2=1;

vecSw=[]; vecKSw=[];

for jjj=1:length(vec_w),
    
puls=vec_w(jjj);
zn=frxav(ZNtf,puls);
zd=frxav(ZDtf,puls);
X=frxav(controller_valid,puls); 


    
z1=X*zn+zd;
Q=inv(X'*X*Wr1'*Wr1+Wr2'*Wr2);

a11c=real(zd'*zd*Wl2'*Wl2+zn'*zn*Wl1'*Wl1);
a11g=real(-Q*z1'*z1);
a11t=-R;

a12c=real(zd'*Wl2'*Wl2);
a12g=real(-Q*z1');
a12t=R*theta_hat;

a22c=real(Wl2'*Wl2);
a22g=real(-Q);
a22t=1-theta_hat'*R*theta_hat;

setlmis([]);
[gamma,ngamma,sgamma]=lmivar(1,[1 0]);
[tau,ntau,stau]=lmivar(1,[1 0]);




lmiterm([-1 1 1 tau],1,1);

lmiterm([-2 1 1 gamma],1,1);

lmiterm([3 1 1 0],a11c);
lmiterm([3 1 1 gamma],a11g,1);
lmiterm([3 1 1 tau],a11t,1); 

lmiterm([3 1 2 0],a12c);
lmiterm([3 1 2 gamma],a12g,1);
lmiterm([3 1 2 tau],a12t,1);

lmiterm([3 2 2 0],a22c);
lmiterm([3 2 2 gamma],a22g,1);
lmiterm([3 2 2 tau],a22t,1);

kappaopt=getlmis;

c=[1;0];


flag=1;
options=[1e-12,1000,1e12,10,flag];
[coutopt,xopt]=mincx(kappaopt,c,options);


if isempty(xopt),
gammaopt=1e12;
else
gammaopt=dec2mat(kappaopt,xopt,1);
end;




vecSw=[vecSw;sqrt(gammaopt)];

vecKSw=[vecKSw;abs(X)*sqrt(gammaopt)];

end;



worstcase_perf=[vec_w, vecTw  , vecKSw , vecGSw , vecSw ];



