
addpath MatlabfunctionsOED

clear
close all

%%% number of data in the identification experiment


ndata=1000;

%%% Definition of the system
Ts=1;
typemodel='arx';
theta_0=[-1.99185 2.20265 -1.84083 0.89413 0.10276 0.18123]';
vector_order=[4 2 3];
var_e=0.5;

[Go,Ho]=determinationGH(typemodel,vector_order,Ts,theta_0);


%%% For simplicity: initial estimate = true system
Gi=Go; Hi=Ho; var_e_i=var_e; theta_init=theta_0;

load initialcontroller %%%% specification of the initial estimate K_init of \hat K
Kinit=ssbal(ss(Kinit));
Kinit.Ts=Ts;



%%%% Specification of either open-loop or closed-loop operation
%%%% identification


%controller=0;  %%% open-loop operation
controller=Kinit; %%%% closed-loop operation (for simplicity the controller in the loop is the same as Kinit)
controller.Ts=Ts;

%%% Specification on the type of spectrum

typespectrum='whitenoise';  % alternative : typespectrum='multisinus';


if strcmp(typespectrum,'whitenoise')
parameter_spectrum=[10]; %specification of M
else
parameter_spectrum=([ 0.1 0.15 0.2 0.25 0.35 0.394  0.46  0.55 0.65 0.75 0.85 0.95 1.05 1.15 1.25 1.35 1.48 1.65 2 3]')/Ts;%specification of the vector of frequencies

end;

%%% Specification on the type of least costly experiment design (cost and
%%% accuracy constraints)

%%% specification of the cost

typecost='sum'; % alternative : typecost='power_r';

if strcmp(typecost,'sum'),   
weighting_cost=[1,1];   parameter_cost=weighting_cost;
elseif strcmp(typecost,'power_r'),
parameter_cost=[];
end;
    
%%% specification of the accuracy constraint


Wl2=tf([0.5165 -0.4632],[1 -0.999455],Ts);  %%% specification of the weighting functions 
Wr2=tf(1,1,Ts);
Wl1=0.1*tf(1,1,Ts); 
Wr1=tf(0,1,Ts);


wbis=logspace(-3,pi);
w_freqweighting=[wbis(1:36),0.35:0.01:0.45,wbis(39:45),1.48,wbis(48:50)]/Ts; %%% specification of Omega

frequency_weighting=[w_freqweighting', frxav(Wl1,w_freqweighting).' , frxav(Wl2,w_freqweighting).' , frxav(Wr1,w_freqweighting).' , frxav(Wr2,w_freqweighting).']; %%% Specification of frequency_weighting.   
%%% Note that frxav(G,vecw) is a function that computes the frequency
%%% response of G(z) at the frequencies in vecw 



chi=chi2inv(0.95,length(theta_0));  %%% specification of \chi in the expression of the uncertainty ellipsoid


    
%%% Computation of the optimal spectrum

[PHI,Pthetath,cost]=LCexpdesign_Id4C(typecost,parameter_cost,frequency_weighting,Kinit,chi,typespectrum,parameter_spectrum,ndata,typemodel,vector_order,Ts,controller,theta_init,var_e_i);



%%% Representation of the optimal spectrum and realization of the optimal
%%% signal

t=0:(ndata-1); t=t'; 


if strcmp(typespectrum,'whitenoise'),

vecw=logspace(-3,pi,200)/Ts;

[phirw]=analysiswhitenoisespectrum(PHI,vecw,Ts);   %%% representation of the spectrum as a function of the frequency

figure(1)
loglog(vecw,phirw)
title('Optimal spectrum')
xlabel('\omega')
ylabel('\Phi_r(\omega)')

r=real_filtwhitenoise(PHI,ndata); %% realization of the optimal signal as a filtered white noise

figure(2)
plot(t,r)
title('Realization of the optimal signal')
xlabel('Samples t')
ylabel('r(t)')

else 

figure(1)
stem(parameter_spectrum,(PHI))
title('Amplitudes of the optimal spectrum')  %%% representation of the the different amplitudes of the multisine as a function of the frequency
xlabel('\omega')
ylabel('A_m')



r=zeros(size(t));
for jjj=1:length(parameter_spectrum),
    r=r+PHI(jjj)*cos(parameter_spectrum(jjj)*t*Ts);   %% realization of the optimal signal as a multisine

end;

figure(2)
plot(t,r)
title('Realization of the optimal signal')
xlabel('Samples t')
ylabel('r(t)')
    
end;


%%% analysis of the robustness 

vecw=logspace(-3,pi,200)/Ts;

[nominal_perf,worstcase_perf,stability]=robustnessanalysis(typemodel,vector_order,Ts,theta_init,Kinit,chi,Pthetath,vecw');  

%%% For each entry Rij of the matrix R, we represent the nominal
%%% performance (blue), the worst case performance (red). For those Rij
%%% that are constrained, we also represent these constraints (black dots). Finally, we also represent \mu(\omega) for the robust stability analysis. 
%%% Note that the analysis is done based on the initial estimates theta_init and Kinit of \hat \theta_N and \hat K that have been used in the optimal experiment design problem 
 
figure(3)
loglog(nominal_perf(:,1),nominal_perf(:,2),'b',worstcase_perf(:,1),worstcase_perf(:,2),'r')
title('R_{11}')

figure(4)
loglog(nominal_perf(:,1),nominal_perf(:,3),'b',worstcase_perf(:,1),worstcase_perf(:,3),'r')
title('R_{21}')

figure(5)
loglog(nominal_perf(:,1),nominal_perf(:,4),'b',worstcase_perf(:,1),worstcase_perf(:,4),'r',worstcase_perf(:,1),1./(abs(frxav(Wl1*Wr2,worstcase_perf(:,1)))),'k.')
title('R_{12}')


figure(6)
loglog(nominal_perf(:,1),nominal_perf(:,5),'b',worstcase_perf(:,1),worstcase_perf(:,5),'r',worstcase_perf(:,1),1./(abs(frxav(Wl2*Wr2,worstcase_perf(:,1)))),'k.')
title('R_{22}')

figure(7)
semilogx(stability(:,1),stability(:,2),'r',stability(:,1),ones(length(stability(:,1)),1),'k.')
title('Robust stability: \mu(\omega)');
axis([stability(1,1),stability(end,1),0,1.1]);


