
addpath MatlabfunctionsOED

clear
close all

%%% Definition of the system
Ts=1;
typemodel='bj';
theta_0=[3.6 -0.9 -0.7]';
vector_order=[1,1,0,1,1];
var_e=1;




[Go,Ho]=determinationGH(typemodel,vector_order,Ts,theta_0);


%%% For simplicity: initial estimate = true system
Gi=Go; Hi=Ho; var_e_i=var_e; theta_init=theta_0;

ndata=500;


Cid=tf(0.1,1,Ts);


controller=0;  %%% open-loop operation
%controller=Cid; %%%% closed-loop operation


%%% Specification on the type of spectrum

typespectrum='whitenoise';  % alternative : typespectrum='multisinus';


if strcmp(typespectrum,'whitenoise')
parameter_spectrum=[20]; %specification of M
else
parameter_spectrum=([0.01 0.05 0.1 0.15 0.2 0.25 0.35 0.45 0.55 0.65 0.75 0.85 0.95 1.05 1.15 1.25 1.35 1.65 2 3]')/Ts;  %specification of the vector of frequencies
end;

%%% Specification on the type of least costly experiment design (cost and
%%% accuracy constraints)

%%% specification of the cost

typecost='power_r'; % alternative : typecost='sum';

if strcmp(typecost,'sum'),   
weighting_cost=[1,1];   parameter_cost=weighting_cost;
elseif strcmp(typecost,'power_r'),
parameter_cost=[];
end;
    
%%% specification of the accuracy constraint

typeaccuracy='r1'; % alternatives : typeoptimality='Radm' or typeoptimality='r2'  

if strcmp(typeaccuracy,'Radm'),
Radm=diag(1./((theta_0/10).^2)); parameter_accuracy=Radm; %%specification of Radm
elseif strcmp(typeaccuracy,'r1') || strcmp(typeaccuracy,'r2'),
Omega=logspace(-3,pi)/Ts; %% choose the set Omega of frequencies at which the accuracy constraint has to be satified and define r_adm(omega) 
parameter_accuracy=[];
for iii=1:length(Omega),
puls=Omega(iii);
gmodw=frxav(Go,puls);
radm=(abs(gmodw)*0.01)/2.45;  
parameter_accuracy=[parameter_accuracy;puls,radm];
end;
end;   
    
%%% Computation of the optimal spectrum

[PHI,Pthetath,cost]=LCexpdesign_simple(typecost,parameter_cost,typeaccuracy,parameter_accuracy,typespectrum,parameter_spectrum,ndata,typemodel,vector_order,Ts,controller,theta_init,var_e_i);

%%% Representation of the optimal spectrum and realization of the optimal
%%% signal

t=0:(ndata-1); t=t'; 


if strcmp(typespectrum,'whitenoise'),

vecw=logspace(-3,pi,200)/Ts;

[phirw]=analysiswhitenoisespectrum(PHI,vecw,Ts);   %%% representation of the spectrum as a function of the frequency

figure(1)
semilogx(vecw,phirw)
title('Optimal spectrum')
xlabel('\omega')
ylabel('\Phi_r(\omega)')

r=real_filtwhitenoise(PHI,ndata); %% realization of the optimal signal as a filtered white noise

figure(2)
plot(t,r)
title('Realization of the optimal signal')
xlabel('Samples t')
ylabel('r(t)')

else 

figure(1)
stem(parameter_spectrum,(PHI))
title('Amplitudes of the optimal spectrum')  %%% representation of the the different amplitudes of the multisine as a function of the frequency
xlabel('\omega')
ylabel('A_m')



r=zeros(size(t));
for jjj=1:length(parameter_spectrum),
    r=r+PHI(jjj)*cos(parameter_spectrum(jjj)*t*Ts);   %% realization of the optimal signal as a multisine

end;

figure(2)
plot(t,r)
title('Realization of the optimal signal')
xlabel('Samples t')
ylabel('r(t)')
    
end;

